package com.hys.app.controller.erp.use;

import com.alibaba.fastjson.JSON;
import com.hys.app.model.support.LogClient;
import com.hys.app.model.support.validator.annotation.Log;
import com.hys.app.service.kuaidiniao.KuaiDiNiaoService;
import com.hys.app.service.kuaidiniao.entity.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

/**
 * OAuth2.0 授权 API
 *
 * @author 张崧
 * @since 2024-02-20
 */
@Api(tags = "快递鸟")
@RestController
@RequestMapping("/admin/erp/kdn")
@Validated
@Slf4j
@AllArgsConstructor
public class KdnController {
    private KuaiDiNiaoService kuaiDiNiaoService;

    @PostMapping("/returnBack")
    public resReturnBackBase kdnReturnBack(ReturnBackBase returnBackBase) {
        return kuaiDiNiaoService.kdnReturnBack(returnBackBase);
    }

    @ApiOperation(value = "快递鸟发货获取单号")
    @PostMapping("/kdShip/{orderId}")
    @Log(client = LogClient.admin, detail = "订单[${orderId}]发货")
    public String kdShip(@PathVariable(value = "orderId") Long orderId
            , @RequestBody @Valid PreOrderModel preOrderModel
            , @RequestBody AddressDTO addressDTO) {
        return kuaiDiNiaoService.kdShip(preOrderModel, orderId, addressDTO);
    }


    @ApiOperation(value = "快递鸟创建单号")
    @PostMapping("/createStore")
    @Log(client = LogClient.admin, detail = "快递鸟创建店铺")
    public void createStore(@RequestBody @Valid CreateStoreModel createStoreModel) {
         kuaiDiNiaoService.createStore(createStoreModel);
    }
}
